@@ -226,6 +226,7 @@ static PlaylistTypes dual_types[] = {
 	PLAYLIST_TYPE2 ("audio/vnd.rn-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
 	PLAYLIST_TYPE2 ("audio/x-realaudio", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
 	PLAYLIST_TYPE2 ("text/plain", totem_pl_parser_add_ra, totem_pl_parser_is_uri_list),
+	PLAYLIST_TYPE2 ("application/x-php", NULL, NULL),
 	PLAYLIST_TYPE2 ("audio/x-ms-asx", totem_pl_parser_add_asx, totem_pl_parser_is_asx),
 	PLAYLIST_TYPE2 ("video/x-ms-asf", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
 	PLAYLIST_TYPE2 ("video/x-ms-wmv", totem_pl_parser_add_asf, totem_pl_parser_is_asf),
@@ -719,6 +720,7 @@ my_g_file_info_get_mime_type_with_data (GFile *file, gpointer *data, TotemPlPars
 
 	*data = NULL;
 
+#ifndef _WIN32
 	/* Stat for a block device, we're screwed as far as speed
 	 * is concerned now */
 	if (g_file_is_native (file) != FALSE) {
@@ -732,6 +734,7 @@ my_g_file_info_get_mime_type_with_data (GFile *file, gpointer *data, TotemPlPars
 		}
 		g_free (path);
 	}
+#endif
 
 	/* Open the file. */
 	stream = g_file_read (file, NULL, &error);
@@ -1125,6 +1128,11 @@ totem_pl_parser_save (TotemPlParser      *parser,
         g_return_val_if_fail (G_IS_FILE (dest), FALSE);
 
         if (totem_pl_playlist_size (playlist) == 0) {
+		/* FIXME add translation */
+		g_set_error (error,
+			     TOTEM_PL_PARSER_ERROR,
+			     TOTEM_PL_PARSER_ERROR_EMPTY_PLAYLIST,
+			     "Playlist selected for saving is empty");
                 return FALSE;
         }
 
@@ -1712,6 +1720,25 @@ totem_pl_parser_ignore_from_mimetype (TotemPlParser *parser, const char *mimetyp
 	return FALSE;
 }
 
+static PlaylistCallback
+totem_pl_parser_get_function_for_mimetype (const char *mimetype)
+{
+	guint i;
+
+	if (mimetype == NULL)
+		return NULL;
+
+	for (i = 0; i < G_N_ELEMENTS(special_types); i++) {
+		if (strcmp (special_types[i].mimetype, mimetype) == 0)
+			return special_types[i].func;
+	}
+	for (i = 0; i < G_N_ELEMENTS(dual_types); i++) {
+		if (strcmp (dual_types[i].mimetype, mimetype) == 0)
+			return dual_types[i].func;
+	}
+	return NULL;
+}
+
 TotemPlParserResult
 totem_pl_parser_parse_internal (TotemPlParser *parser,
 				GFile *file,
@@ -1782,7 +1809,7 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 	DEBUG(file, g_print ("_get_mime_type_for_name for '%s' returned '%s'\n", uri, mimetype));
 	if (mimetype == NULL || strcmp (UNKNOWN_TYPE, mimetype) == 0
 	    || (g_file_is_native (file) && g_content_type_is_a (mimetype, "text/plain") != FALSE)) {
-	    	char *new_mimetype;
+		char *new_mimetype;
 		new_mimetype = my_g_file_info_get_mime_type_with_data (file, &data, parser);
 		if (new_mimetype) {
 			g_free (mimetype);
@@ -1852,6 +1879,8 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 
 		for (i = 0; i < G_N_ELEMENTS(dual_types) && found == FALSE; i++) {
 			if (strcmp (dual_types[i].mimetype, mimetype) == 0) {
+				PlaylistCallback func;
+
 				DEBUG(file, g_print ("URI '%s' is dual type '%s'\n", uri, mimetype));
 				if (data == NULL) {
 					g_free (mimetype);
@@ -1865,13 +1894,24 @@ totem_pl_parser_parse_internal (TotemPlParser *parser,
 					mimetype = NULL;
 					break;
 				}
+				/* Now look for the proper function to use */
+				func = totem_pl_parser_get_function_for_mimetype (mimetype);
+				if (func == NULL && mimetype != NULL) {
+					DEBUG(file, g_print ("Ignoring URI '%s' because we couldn't find a playlist parser for '%s'\n", uri, mimetype));
+					ret = TOTEM_PL_PARSER_RESULT_IGNORED;
+					g_free (mimetype);
+					mimetype = NULL;
+					break;
+				} else if (func == NULL) {
+					func = dual_types[i].func;
+				}
 
 				if (base_file == NULL)
 					base_file = g_file_get_parent (file);
 				else
 					base_file = g_object_ref (base_file);
 
-				ret = (* dual_types[i].func) (parser, file, base_file ? base_file : file, parse_data, data);
+				ret = (* func) (parser, file, base_file ? base_file : file, parse_data, data);
 
 				if (base_file != NULL)
 					g_object_unref (base_file);
@@ -2295,6 +2335,8 @@ totem_pl_parser_mime_type_from_data (gconstpointer data, int len)
 			if (func == dual_types[i].iden)
 				continue;
 			func = dual_types[i].iden;
+			if (func == NULL)
+				continue;
 			res = func (data, len);
 			if (res != NULL) {
 				g_free (mime_type);
